@@ -497,12 +497,14 @@ verify_test(void)
 {
 	unit_show_feature("signature verify");
 	verifytest_file("testdata/test_signatures.1", "20070818005004");
+#ifdef USE_DSA
 	verifytest_file("testdata/test_signatures.2", "20080414005004");
 	verifytest_file("testdata/test_signatures.3", "20080416005004");
 	verifytest_file("testdata/test_signatures.4", "20080416005004");
 	verifytest_file("testdata/test_signatures.5", "20080416005004");
 	verifytest_file("testdata/test_signatures.6", "20080416005004");
 	verifytest_file("testdata/test_signatures.7", "20070829144150");
+#endif /* USE_DSA */
 	verifytest_file("testdata/test_signatures.8", "20070829144150");
 #if (defined(HAVE_EVP_SHA256) || defined(HAVE_NSS) || defined(HAVE_NETTLE)) && defined(USE_SHA2)
 	verifytest_file("testdata/test_sigs.rsasha256", "20070829144150");
