@@ -565,7 +565,7 @@ win_do_cron(void* ATTR_UNUSED(arg))
 
 /** Set the timer for cron for the next wake up */
 static void
-set_cron_timer()
+set_cron_timer(void)
 {
 	struct timeval tv;
 	int crontime;
